from enum import Enum
from typing import Tuple


def value_value(enumeration: Enum) -> Tuple[str, str]:
    return (
        str(enumeration.value),
        str(enumeration.value)
    )


def attribute_attribute(enumeration: Enum) -> Tuple[str, str]:
    return (
        str(enumeration.name),
        str(enumeration.name)
    )


def attribute_value(enumeration: Enum) -> Tuple[str, str]:
    return (
        str(enumeration.name),
        str(enumeration.value)
    )


def value_attribute(enumeration: Enum) -> Tuple[str, str]:
    return (
        str(enumeration.value),
        str(enumeration.name)
    )



from django.apps import apps
from django.db import models
from django.contrib.postgres import fields as pg_fields


POSTGRES = 'postgresql'
DEFAULT = 'default'


class DataBaseRouter:
    def _get_postgresql_fields(self):
        return [
            var for var in vars(pg_fields).values()
            if isinstance(var, type) and issubclass(var, models.Field)
        ]

    def _get_field_classes(self, db_obj):
        return [
            type(field) for field in db_obj._meta.get_fields()
        ]

    def has_postgres_field(self, db_obj):
        field_classes = self._get_field_classes(db_obj)

        return len([
            field_cls for field_cls in field_classes
            if field_cls in self._get_postgresql_fields()
        ]) > 0

    def db_for_read(self, model, **hints):
        if self.has_postgres_field(model):
            return POSTGRES

        return DEFAULT

    def db_for_write(self, model, **hints):
        if self.has_postgres_field(model):
            return POSTGRES

        return DEFAULT

    def allow_relation(self, obj1, obj2, **hints):
        if not self.has_postgres_field(obj1) and not self.has_postgres_field(obj2):
            return True

        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        if model_name is not None and \
           db == DEFAULT and \
           self.has_postgres_field(apps.get_model(app_label, model_name)):
            return False

        return True



from enum import Enum


class CharTestEnum(Enum):
    FIRST = 'first'
    SECOND = 'second'
    THIRD = 'third'


class CharLongValuesTestEnum(Enum):
    FIRST = 'first value'
    SECOND = 'second value'
    THIRD = 'third value'


class IntTestEnum(Enum):
    FIRST = 1
    SECOND = 2
    THIRD = 3



from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers
from rest_framework.utils.field_mapping import get_field_kwargs

from .fields import EnumChoiceField as ModelEnumChoiceField
from .choice_builders import value_value
from .utils import as_choice_builder, value_from_built_choice

NO_KEY_MSG = _('Key {failing_key} is not a valid {enum_class_name}')
NOT_A_LIST_MSG = _('Expected a list of items but got type "{input_type}".')
EMPTY_MSG = _('This selection may not be empty.')


class EnumChoiceField(serializers.Field):
    default_error_messages = {
        'non_existent_key': NO_KEY_MSG
    }

    def __init__(self, enum_class, choice_builder=value_value, **kwargs):
        super().__init__(**kwargs)
        self.enum_class = enum_class
        self.choice_builder = as_choice_builder(choice_builder)

    def to_representation(self, value):
        return value_from_built_choice(
            self.choice_builder(value)
        )

    def to_internal_value(self, value):
        for choice in self.enum_class:
            if value_from_built_choice(self.choice_builder(choice)) == value:
                return choice

        self.fail(
            'non_existent_key',
            failing_key=value,
            enum_class_name=self.enum_class.__name__
        )


class MultipleEnumChoiceField(EnumChoiceField):
    default_error_messages = {
        'non_existent_key': NO_KEY_MSG,
        'not_a_list': NOT_A_LIST_MSG,
        'empty': EMPTY_MSG
    }

    def __init__(self, *args, **kwargs):
        self.allow_empty = kwargs.pop('allow_empty', False)

        super().__init__(*args, **kwargs)

        self.default_error_messages = {
            **self.default_error_messages,

        }

    def to_internal_value(self, data):
        if not isinstance(data, list):
            self.fail('not_a_list', input_type=type(data).__name__)

        if not self.allow_empty and not data:
            self.fail('empty')

        return [
            super(MultipleEnumChoiceField, self).to_internal_value(value)
            for value in data
        ]

    def to_representation(self, data):
        return [
            super(MultipleEnumChoiceField, self).to_representation(value)
            for value in data
        ]


class EnumChoiceModelSerializerMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.serializer_field_mapping[ModelEnumChoiceField] = EnumChoiceField

    def build_standard_field(self, field_name, model_field):
        """
        By default `ModelSerializer.build_standard_field` coerces any field
        that has a model field with choices to `ChoiceField` wich returns the
        value directly.

        Since enum values resemble `EnumClass.ENUM_INSTANCE`
        they won't be able to be encoded by the JSONEncoder when being passed
        to a `Response`.
        """

        if isinstance(model_field, ModelEnumChoiceField):
            # These are kwargs, generated by `get_field_kwargs`
            # but are not needed for our field.
            # `model_field` is used only in children of DRF's `ModelField`
            # `choices` is not used because we use `field.enum_class` to validate the choice
            # `max_length` is generated from the model field's max_length and we don't use it
            dump_kwargs = ('model_field', 'choices', 'max_length', 'allow_blank')

            initial_kwargs = {
                'enum_class': model_field.enum_class,
                'choice_builder': model_field.choice_builder,
                **get_field_kwargs(field_name, model_field)
            }
            finalized_kwargs = {
                key: value for key, value in initial_kwargs.items()
                if key not in dump_kwargs
            }

            return EnumChoiceField, finalized_kwargs

        return super().build_standard_field(field_name, model_field)



import django_filters as filters

from django_enum_choices.filters import EnumChoiceFilter, EnumChoiceFilterSetMixin

from .enumerations import MyEnum
from .models import MyModel
from .choice_builders import custom_choice_builder


class ExplicitFilterSet(filters.FilterSet):
    enumerated_field = EnumChoiceFilter(MyEnum)


class ExplicitCustomChoiceBuilderFilterSet(filters.FilterSet):
    enumerated_field = EnumChoiceFilter(
        MyEnum,
        choice_builder=custom_choice_builder
    )


class ImplicitFilterSet(EnumChoiceFilterSetMixin, filters.FilterSet):
    class Meta:
        model = MyModel
        fields = ['enumerated_field']



from typing import Callable, Tuple, Any
from enum import Enum

from django.utils.translation import gettext as _

from .exceptions import EnumChoiceFieldException


def as_choice_builder(choice_builder):
    def inner(enumeration):
        if not enumeration:
            return enumeration

        built = choice_builder(enumeration)

        return tuple(str(value) for value in built)

    return inner


def value_from_built_choice(built_choice):
    if isinstance(built_choice, tuple):
        return built_choice[0]

    return built_choice


def validate_built_choices(
    enum_class: Enum,
    built_choices: Tuple[Tuple[Any]]
):
    MEMBER_KEY = 'key'
    MEMBER_VALUE = 'value'

    message = _(
        'Received type {failing_type} on {failing_member} inside choice: {failing_choice}.\n' +
        'All choices generated from {failing_enum_class} must be strings.'
    )

    for key, value in built_choices:
        message_kwargs = {
            'failing_choice': (key, value),
            'failing_enum_class': enum_class
        }

        if not isinstance(key, str):
            message_kwargs.update({
                'failing_type': type(key),
                'failing_member': MEMBER_KEY
            })

            raise EnumChoiceFieldException(
                message.format(**message_kwargs)
            )

        if not isinstance(value, str):
            message_kwargs.update({
                'failing_type': type(value),
                'failing_member': MEMBER_VALUE
            })

            raise EnumChoiceFieldException(
                message.format(**message_kwargs)
            )


def build_enum_choices(
    enum_class: Enum,
    choice_builder: Callable
) -> Tuple[Tuple[str]]:
    choices = [
        choice_builder(choice)
        for choice in enum_class
    ]

    validate_built_choices(enum_class, choices)

    return choices



from __future__ import unicode_literals

import environ

env = environ.Env()

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:"
    },
    'postgresql': env.db('DATABASE_URL', default='postgres:///django_enum_choices')

}

DATABASE_ROUTERS = ['tests.testapp.database_routers.DataBaseRouter']

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "tests.testapp.apps.TestAppConfig"
]

SITE_ID = 1

SECRET_KEY = "test"



from django import forms

from django_enum_choices.forms import EnumChoiceField

from .models import MyModel
from .enumerations import MyEnum
from .choice_builders import custom_choice_builder


class StandardEnumForm(forms.Form):
    enumerated_field = EnumChoiceField(MyEnum)


class ModelEnumForm(forms.ModelForm):
    class Meta:
        model = MyModel
        fields = ['enumerated_field']


class CustomChoiceBuilderEnumForm(forms.Form):
    enumerated_field = EnumChoiceField(
        MyEnum,
        choice_builder=custom_choice_builder
    )



from django.test import TestCase

from django_enum_choices.forms import EnumChoiceField

from .testapp.enumerations import CharTestEnum


class FormFieldTests(TestCase):
    def test_field_instance_creates_choices_correctly(self):
        instance = EnumChoiceField(CharTestEnum)
        choices = instance.build_choices()

        self.assertEqual(
            choices,
            [('first', 'first'),
             ('second', 'second'),
             ('third', 'third')]
        )

    def test_field_instance_creates_choices_correctly_with_custom_choice_builder(self):
        def choice_builder(choice):
            return 'Custom_' + choice.value, choice.value

        instance = EnumChoiceField(CharTestEnum, choice_builder=choice_builder)
        choices = instance.build_choices()

        self.assertEqual(
            choices,
            [('Custom_first', 'first'),
             ('Custom_second', 'second'),
             ('Custom_third', 'third')]
        )



from django.apps import apps

if apps.apps_ready:
    """

    `register_enum_choice_list_filter` depends on `django.conf.settings`
    The tests do not define a settings module so we need to execute
    `register_enum_choice_list_filter` only when the apps are loaded.

    """

    from .admin import register_enum_choice_list_filter

    register_enum_choice_list_filter()



# Generated by Django 2.2.2 on 2019-07-04 11:24

import django.contrib.postgres.fields
from django.db import migrations, models
import django_enum_choices.fields
import examples.enumerations


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MyModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enumerated_field', django_enum_choices.fields.EnumChoiceField(choices=[('a', 'a'), ('b', 'b')], enum_class=examples.enumerations.MyEnum, max_length=1)),
            ],
        ),
        migrations.CreateModel(
            name='MyModelMultiple',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enumerated_field', django.contrib.postgres.fields.ArrayField(base_field=django_enum_choices.fields.EnumChoiceField(choices=[('a', 'a'), ('b', 'b')], enum_class=examples.enumerations.MyEnum, max_length=1), size=None)),
            ],
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-19 12:38

from django.db import migrations, models
import django_enum_choices.fields
import examples.enumerations
import examples.models


class Migration(migrations.Migration):

    dependencies = [
        ('examples', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomReadableValueEnumModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enumerated_field', django_enum_choices.fields.EnumChoiceField(choice_builder=examples.models.choice_builder, choices=[('a', 'A'), ('b', 'B')], enum_class=examples.enumerations.MyEnum, max_length=1)),
            ],
        ),
    ]



from django.test import TestCase

from rest_framework.exceptions import ValidationError

from django_enum_choices.serializers import EnumChoiceField, MultipleEnumChoiceField
from .testapp.enumerations import IntTestEnum, CharTestEnum


class TestSerializerField(TestCase):
    def test_to_representation_returns_primitive_int_value(self):
        field = EnumChoiceField(enum_class=IntTestEnum)

        result = field.to_representation(IntTestEnum.FIRST)

        self.assertEqual(result, '1')

    def test_to_representation_returns_primitive_string_value(self):
        field = EnumChoiceField(enum_class=CharTestEnum)

        result = field.to_representation(CharTestEnum.FIRST)

        self.assertEqual(result, 'first')

    def test_to_internal_value_fails_when_value_not_in_enum_class(self):
        failing_value = 5
        field = EnumChoiceField(enum_class=IntTestEnum)

        with self.assertRaisesMessage(
            ValidationError,
            'Key 5 is not a valid IntTestEnum'
        ):
            field.to_internal_value(failing_value)

    def test_to_internal_value_returns_enum_value_when_value_is_int(self):
        field = EnumChoiceField(enum_class=IntTestEnum)

        result = field.to_internal_value('1')

        self.assertEqual(result, IntTestEnum.FIRST)

    def test_to_internal_value_returns_enum_value_when_value_is_string(self):
        field = EnumChoiceField(enum_class=CharTestEnum)

        result = field.to_internal_value('first')

        self.assertEqual(result, CharTestEnum.FIRST)


class TestMultipleSerializerField(TestCase):
    def test_to_representation_returns_list_of_ints(self):
        field = MultipleEnumChoiceField(enum_class=IntTestEnum)

        result = field.to_representation([IntTestEnum.FIRST, IntTestEnum.SECOND])

        self.assertEqual(['1', '2'], result)

    def test_to_representation_returns_list_of_strings(self):
        field = MultipleEnumChoiceField(enum_class=CharTestEnum)

        result = field.to_representation([CharTestEnum.FIRST, CharTestEnum.SECOND])

        self.assertEqual(['first', 'second'], result)

    def test_to_internal_value_fails_when_value_is_not_list(self):
        field = MultipleEnumChoiceField(enum_class=IntTestEnum)

        with self.assertRaisesMessage(
            ValidationError,
            'Expected a list of items but got type "int".'
        ):
            field.to_internal_value(5)

    def test_to_internal_value_fails_when_not_allowed_empty_field_gets_empty_list(self):
        field = MultipleEnumChoiceField(enum_class=IntTestEnum)

        with self.assertRaisesMessage(
            ValidationError,
            'This selection may not be empty.'
        ):
            field.to_internal_value([])

    def test_to_internal_value_returns_list_of_enums_when_value_is_list_of_ints(self):
        field = MultipleEnumChoiceField(enum_class=IntTestEnum)

        result = field.to_internal_value(['1', '2'])

        self.assertEqual([IntTestEnum.FIRST, IntTestEnum.SECOND], result)

    def test_to_internal_value_returns_list_of_enums_when_value_is_list_of_strings(self):
        field = MultipleEnumChoiceField(enum_class=CharTestEnum)

        result = field.to_internal_value(['first', 'second'])

        self.assertEqual([CharTestEnum.FIRST, CharTestEnum.SECOND], result)



from django.test import TestCase

from rest_framework import serializers

from django_enum_choices.serializers import (
    EnumChoiceField,
    EnumChoiceModelSerializerMixin,
    MultipleEnumChoiceField
)
from .testapp.models import (
    StringEnumeratedModel,
    MultipleEnumeratedModel,
    CustomChoiceBuilderEnumeratedModel,
    BlankNullableEnumeratedModel
)
from .testapp.enumerations import CharTestEnum


class EnumChoiceFieldSerializerIntegrationTests(TestCase):
    class Serializer(serializers.Serializer):
        enumeration = EnumChoiceField(enum_class=CharTestEnum)

    def test_field_value_is_serialized_correctly(self):
        serializer = self.Serializer({'enumeration': CharTestEnum.FIRST})

        result = serializer.data['enumeration']

        self.assertEqual(result, 'first')

    def test_field_is_deserialized_correctly(self):
        serializer = self.Serializer(data={'enumeration': 'first'})
        serializer.is_valid()

        result = serializer.validated_data['enumeration']

        self.assertEqual(result, CharTestEnum.FIRST)

    def test_serializer_is_not_valid_when_field_is_required_and_not_provided(self):
        class Serializer(serializers.Serializer):
            enumeration = EnumChoiceField(
                enum_class=CharTestEnum
            )

        serializer = Serializer(data={})

        self.assertFalse(serializer.is_valid())

    def test_serializer_is_valid_when_field_is_required_and_provided(self):
        class Serializer(serializers.Serializer):
            enumeration = EnumChoiceField(
                enum_class=CharTestEnum
            )

        serializer = Serializer(data={'enumeration': 'first'})

        self.assertTrue(serializer.is_valid)

    def test_serializer_does_not_add_field_to_validated_data_when_not_required_and_not_provided(self):
        class Serializer(serializers.Serializer):
            enumeration = EnumChoiceField(
                enum_class=CharTestEnum,
                required=False
            )

        serializer = Serializer(data={})
        is_valid = serializer.is_valid()

        self.assertTrue(is_valid)
        self.assertNotIn(
            'enumeration',
            serializer.validated_data.keys()
        )

    def test_serializer_adds_field_to_validated_data_when_not_required_and_provided(self):
        class Serializer(serializers.Serializer):
            enumeration = EnumChoiceField(
                enum_class=CharTestEnum,
                required=False
            )

        serializer = Serializer(data={'enumeration': 'first'})
        is_valid = serializer.is_valid()

        self.assertTrue(is_valid)
        self.assertIn(
            'enumeration',
            serializer.validated_data.keys()
        )

    def test_serializer_is_not_valid_when_field_is_not_nullable_and_value_is_none(self):
        class Serializer(serializers.Serializer):
            enumeration = EnumChoiceField(
                enum_class=CharTestEnum
            )

        serializer = Serializer(data={'enumeration': None})

        self.assertFalse(serializer.is_valid())

    def test_serializer_is_valid_when_field_is_nullable_and_value_is_none(self):
        class Serializer(serializers.Serializer):
            enumeration = EnumChoiceField(
                enum_class=CharTestEnum,
                allow_null=True
            )

        serializer = Serializer(data={'enumeration': None})

        self.assertTrue(serializer.is_valid())


class MultipleEnumChoiceFieldSerializerIntegrationTests(TestCase):
    class Serializer(serializers.Serializer):
        enumeration = MultipleEnumChoiceField(enum_class=CharTestEnum)

    def test_multiple_field_is_serialized_correctly(self):
        serializer = self.Serializer({
            'enumeration': [CharTestEnum.FIRST, CharTestEnum.SECOND]
        })

        result = serializer.data['enumeration']

        self.assertEqual(result, ['first', 'second'])

    def test_multiple_field_is_deserialized_correctly(self):
        serializer = self.Serializer(
            data={
                'enumeration': [CharTestEnum.FIRST.value, CharTestEnum.SECOND.value]
            }
        )
        self.assertTrue(serializer.is_valid())

        result = serializer.validated_data['enumeration']

        self.assertEqual(result, [CharTestEnum.FIRST, CharTestEnum.SECOND])

    def test_serializer_is_not_valid_when_field_is_required_and_not_provided(self):
        class Serializer(serializers.Serializer):
            enumeration = MultipleEnumChoiceField(
                enum_class=CharTestEnum
            )

        serializer = Serializer(data={})

        self.assertFalse(serializer.is_valid())

    def test_serializer_is_not_valid_when_field_is_required_and_provided_but_not_a_list(self):
        class Serializer(serializers.Serializer):
            enumeration = MultipleEnumChoiceField(
                enum_class=CharTestEnum
            )

        serializer = Serializer(data={'enumeration': 'first'})

        self.assertFalse(serializer.is_valid())

    def test_serializer_is_valid_when_field_is_required_and_provided(self):
        class Serializer(serializers.Serializer):
            enumeration = MultipleEnumChoiceField(
                enum_class=CharTestEnum
            )

        serializer = Serializer(data={'enumeration': ['first']})

        self.assertTrue(serializer.is_valid)

    def test_serializer_does_not_add_field_to_validated_data_when_not_required_and_not_provided(self):
        class Serializer(serializers.Serializer):
            enumeration = MultipleEnumChoiceField(
                enum_class=CharTestEnum,
                required=False
            )

        serializer = Serializer(data={})
        is_valid = serializer.is_valid()

        self.assertTrue(is_valid)
        self.assertNotIn(
            'enumeration',
            serializer.validated_data.keys()
        )

    def test_serializer_adds_field_to_validated_data_when_not_required_and_provided(self):
        class Serializer(serializers.Serializer):
            enumeration = MultipleEnumChoiceField(
                enum_class=CharTestEnum,
                required=False
            )

        serializer = Serializer(data={'enumeration': ['first']})
        is_valid = serializer.is_valid()

        self.assertTrue(is_valid)
        self.assertIn(
            'enumeration',
            serializer.validated_data.keys()
        )

    def test_serializer_is_not_valid_when_field_is_not_nullable_and_value_is_none(self):
        class Serializer(serializers.Serializer):
            enumeration = MultipleEnumChoiceField(
                enum_class=CharTestEnum
            )

        serializer = Serializer(data={'enumeration': None})

        self.assertFalse(serializer.is_valid())

    def test_serializer_is_valid_when_field_is_nullable_and_value_is_none(self):
        class Serializer(serializers.Serializer):
            enumeration = MultipleEnumChoiceField(
                enum_class=CharTestEnum,
                allow_null=True
            )

        serializer = Serializer(data={'enumeration': None})

        self.assertTrue(serializer.is_valid())

    def test_serializer_is_not_valid_when_field_is_not_allow_empty_but_empty_list_is_provided(self):
        class Serializer(serializers.Serializer):
            enumeration = MultipleEnumChoiceField(
                enum_class=CharTestEnum
            )

        serializer = Serializer(data={'enumeration': []})

        self.assertFalse(serializer.is_valid())

    def test_serializer_is_valid_when_field_is_allow_empty_and_empty_list_is_provided(self):
        class Serializer(serializers.Serializer):
            enumeration = MultipleEnumChoiceField(
                enum_class=CharTestEnum,
                allow_empty=True
            )

        serializer = Serializer(data={'enumeration': []})

        self.assertTrue(serializer.is_valid())


class EnumChoiceFieldModelSerializerIntegrationTests(TestCase):
    class Serializer(serializers.ModelSerializer):
        enumeration = EnumChoiceField(enum_class=CharTestEnum)

        class Meta:
            model = StringEnumeratedModel
            fields = ('enumeration', )

    def test_field_value_is_serialized_correctly(self):
        instance = StringEnumeratedModel.objects.create(
            enumeration=CharTestEnum.FIRST
        )

        serializer = self.Serializer(instance)

        result = serializer.data['enumeration']

        self.assertEqual(result, 'first')

    def test_field_is_deserialized_correctly(self):
        serializer = self.Serializer(data={'enumeration': 'first'})
        serializer.is_valid()

        result = serializer.validated_data['enumeration']

        self.assertEqual(result, CharTestEnum.FIRST)

    def test_field_is_serialized_correctly_when_using_serializer_mixin(self):
        class Serializer(EnumChoiceModelSerializerMixin, serializers.ModelSerializer):
            class Meta:
                model = StringEnumeratedModel
                fields = ('enumeration', )

        instance = StringEnumeratedModel.objects.create(
            enumeration=CharTestEnum.FIRST
        )
        serializer = Serializer(instance)
        result = serializer.data['enumeration']

        self.assertEqual('first', result)

    def test_field_is_deserialized_correctly_when_using_serializer_mixin(self):
        class Serializer(EnumChoiceModelSerializerMixin, serializers.ModelSerializer):
            class Meta:
                model = StringEnumeratedModel
                fields = ('enumeration', )

        serializer = self.Serializer(data={'enumeration': 'first'})
        serializer.is_valid()

        result = serializer.validated_data['enumeration']

        self.assertEqual(CharTestEnum.FIRST, result)

    def test_instance_is_created_successfully_after_model_serializer_create(self):
        class Serializer(EnumChoiceModelSerializerMixin, serializers.ModelSerializer):
            class Meta:
                model = StringEnumeratedModel
                fiedls = ('enumeration', )

        current_instance_count = StringEnumeratedModel.objects.count()

        serializer = self.Serializer(data={'enumeration': 'first'})
        serializer.is_valid()

        instance = serializer.create(serializer.validated_data)

        self.assertEqual(
            current_instance_count + 1,
            StringEnumeratedModel.objects.count()
        )
        self.assertEqual(
            CharTestEnum.FIRST,
            instance.enumeration
        )

    def test_instance_is_updated_successfully_after_model_serializer_update(self):
        class Serializer(EnumChoiceModelSerializerMixin, serializers.ModelSerializer):
            class Meta:
                model = StringEnumeratedModel
                fiedls = ('enumeration', )

        instance = StringEnumeratedModel.objects.create(
            enumeration=CharTestEnum.FIRST
        )

        serializer = self.Serializer(data={'enumeration': 'second'})
        serializer.is_valid()

        serializer.update(instance, serializer.validated_data)
        instance.refresh_from_db()

        self.assertEqual(
            CharTestEnum.SECOND,
            instance.enumeration
        )

    def test_instance_is_created_successfully_when_using_custom_choice_builder(self):
        class Serializer(EnumChoiceModelSerializerMixin, serializers.ModelSerializer):
            class Meta:
                model = CustomChoiceBuilderEnumeratedModel
                fields = ('enumeration', )

        current_instance_count = CustomChoiceBuilderEnumeratedModel.objects.count()

        serializer = Serializer(data={'enumeration': 'Custom_first'})
        self.assertTrue(serializer.is_valid())

        instance = serializer.create(serializer.validated_data)

        self.assertEqual(
            current_instance_count + 1,
            CustomChoiceBuilderEnumeratedModel.objects.count()
        )
        self.assertEqual(
            CharTestEnum.FIRST,
            instance.enumeration
        )

    def test_field_can_handle_allow_blank(self):
        # See GH-45
        class Serializer(EnumChoiceModelSerializerMixin, serializers.ModelSerializer):
            class Meta:
                model = BlankNullableEnumeratedModel
                fields = ('enumeration', )

        instance = StringEnumeratedModel.objects.create(
            enumeration=CharTestEnum.FIRST
        )
        serializer = Serializer(instance)
        serializer.data


class MultipleEnumChoiceFieldModelSerializerIntegrationTests(TestCase):
    databases = ['default', 'postgresql']

    class Serializer(EnumChoiceModelSerializerMixin, serializers.ModelSerializer):
        class Meta:
            model = MultipleEnumeratedModel
            fields = ('enumeration', )

    def test_mulitple_field_is_serialized_correctly_when_using_serializer_mixin(self):
        instance = MultipleEnumeratedModel.objects.create(
            enumeration=[CharTestEnum.FIRST, CharTestEnum.SECOND]
        )

        serializer = self.Serializer(instance)
        result = serializer.data['enumeration']

        self.assertEqual(['first', 'second'], result)

    def test_multiple_field_is_deserialized_correctly_when_using_serializer_mixin(self):
        serializer = self.Serializer(data={'enumeration': ['first', 'second']})
        serializer.is_valid()

        result = serializer.validated_data['enumeration']

        self.assertEqual([CharTestEnum.FIRST, CharTestEnum.SECOND], result)

    def test_instance_is_created_successfully_after_model_serializer_create(self):
        current_instance_count = MultipleEnumeratedModel.objects.count()

        serializer = self.Serializer(data={'enumeration': ['first', 'second']})
        serializer.is_valid()

        instance = serializer.create(serializer.validated_data)

        self.assertEqual(
            current_instance_count + 1,
            MultipleEnumeratedModel.objects.count()
        )
        self.assertEqual(
            [CharTestEnum.FIRST, CharTestEnum.SECOND],
            instance.enumeration
        )

    def test_instance_is_updated_successfully_after_model_serializer_update(self):
        instance = MultipleEnumeratedModel.objects.create(
            enumeration=[CharTestEnum.FIRST, CharTestEnum.SECOND]
        )

        serializer = self.Serializer(data={'enumeration': ['first', 'second', 'third']})
        serializer.is_valid()

        serializer.update(instance, serializer.validated_data)
        instance.refresh_from_db()

        self.assertEqual(
            [CharTestEnum.FIRST, CharTestEnum.SECOND, CharTestEnum.THIRD],
            instance.enumeration
        )



from django.test import TestCase

from django_enum_choices.filters import EnumChoiceFilter
from django_enum_choices.forms import EnumChoiceField as EnumChoiceFormField

from .testapp.enumerations import CharTestEnum


class EnumChoiceFilterTests(TestCase):
    def test_filter_instance_extra_has_enum_class_and_choice_builder(self):
        instance = EnumChoiceFilter(enum_class=CharTestEnum)

        self.assertEqual(instance.extra['enum_class'], CharTestEnum)
        self.assertIsNotNone(instance.extra['choice_builder'])

    def test_corresponding_field_is_of_correct_type(self):
        instance = EnumChoiceFilter(enum_class=CharTestEnum)
        form_field = instance.field

        self.assertIsInstance(form_field, EnumChoiceFormField)

    def test_corresponding_field_has_enum_class_and_choice_builder(self):
        instance = EnumChoiceFilter(enum_class=CharTestEnum)

        form_field = instance.field

        self.assertEqual(form_field.enum_class, CharTestEnum)
        self.assertIsNotNone(form_field.choice_builder)



from rest_framework import serializers

from django_enum_choices.serializers import (
    EnumChoiceField,
    MultipleEnumChoiceField,
    EnumChoiceModelSerializerMixin
)


from .enumerations import MyEnum
from .models import MyModel, MyModelMultiple
from .choice_builders import custom_choice_builder


# Standard Serializer
class MySerializer(serializers.Serializer):
    enumerated_field = EnumChoiceField(MyEnum)


# Model Serializer
class MyModelSerializer(serializers.ModelSerializer):
    enumerated_field = EnumChoiceField(MyEnum)

    class Meta:
        model = MyModel
        fields = ('enumerated_field', )


# Model Serializer without field declaration
class ImplicitMyModelSerializer(
    EnumChoiceModelSerializerMixin,
    serializers.ModelSerializer
):
    class Meta:
        model = MyModel
        fields = ('enumerated_field', )


# Multiple Standard Serializer
class MultipleMySerializer(serializers.Serializer):
    enumerated_field = MultipleEnumChoiceField(MyEnum)


# Multiple Model Serializer without field declaration
class ImplicitMultipleMyModelSerializer(
    EnumChoiceModelSerializerMixin,
    serializers.ModelSerializer
):
    class Meta:
        model = MyModelMultiple
        fields = ('enumerated_field', )


# Custom choice builder serializer
class CustomChoiceBuilderSerializer(serializers.Serializer):
    enumerated_field = EnumChoiceField(
        MyEnum,
        choice_builder=custom_choice_builder
    )



from django.test import TestCase

import django_filters as filters
from django_filters import rest_framework as drf_filters

from django_enum_choices.filters import EnumChoiceFilter, EnumChoiceFilterSetMixin

from .testapp.enumerations import CharTestEnum
from .testapp.models import (
    StringEnumeratedModel,
    CustomChoiceBuilderEnumeratedModel,
    custom_choice_builder
)


class FilterSetIntegrationTests(TestCase):
    class ExplicitFilterSet(filters.FilterSet):
        enumeration = EnumChoiceFilter(CharTestEnum)

    class ExplicitChoiceBuilderFilterSet(filters.FilterSet):
        enumeration = EnumChoiceFilter(CharTestEnum, choice_builder=custom_choice_builder)

    class ImplicitFilterSet(EnumChoiceFilterSetMixin, filters.FilterSet):
        class Meta:
            model = StringEnumeratedModel
            fields = ['enumeration']

    class ImplicitChoiceBuilderFilterSet(EnumChoiceFilterSetMixin, filters.FilterSet):
        class Meta:
            model = CustomChoiceBuilderEnumeratedModel
            fields = ['enumeration']

    def setUp(self):
        for choice in CharTestEnum:
            StringEnumeratedModel.objects.create(
                enumeration=choice
            )

    def test_explicitly_declarated_field_filters_correctly(self):
        filters = {
            'enumeration': 'first'
        }
        filterset = self.ExplicitFilterSet(filters, StringEnumeratedModel.objects.all())

        self.assertEqual(filterset.qs.count(), 1)
        self.assertEqual(filterset.qs.first().enumeration, CharTestEnum.FIRST)

    def test_explicitly_declarated_field_filters_correctly_with_custom_choice_builder(self):
        filters = {
            'enumeration': 'Custom_first'
        }
        filterset = self.ExplicitChoiceBuilderFilterSet(filters, StringEnumeratedModel.objects.all())

        self.assertEqual(filterset.qs.count(), 1)
        self.assertEqual(filterset.qs.first().enumeration, CharTestEnum.FIRST)

    def test_filter_by_non_valid_choice_returns_full_queryset(self):
        filters = {
            'enumeration': 'invalid'
        }
        filterset = self.ExplicitChoiceBuilderFilterSet(filters, StringEnumeratedModel.objects.all())

        self.assertEqual(filterset.qs.count(), StringEnumeratedModel.objects.count())

    def test_implicit_filter_filters_correctly(self):
        filters = {
            'enumeration': 'first'
        }
        filterset = self.ImplicitFilterSet(filters)

        self.assertEqual(filterset.qs.count(), 1)
        self.assertEqual(filterset.qs.first().enumeration, CharTestEnum.FIRST)

    def test_implicit_filter_filters_correctly_on_field_with_custom_choice_builder(self):
        for choice in CharTestEnum:
            CustomChoiceBuilderEnumeratedModel.objects.create(enumeration=choice)

        filters = {
            'enumeration': 'Custom_first'
        }
        filterset = self.ImplicitChoiceBuilderFilterSet(filters)

        self.assertEqual(filterset.qs.count(), 1)
        self.assertEqual(filterset.qs.first().enumeration, CharTestEnum.FIRST)


class FilterSetDRFIntegrationTests(TestCase):
    class ExplicitFilterSet(drf_filters.FilterSet):
        enumeration = EnumChoiceFilter(CharTestEnum)

    class ExplicitChoiceBuilderFilterSet(drf_filters.FilterSet):
        enumeration = EnumChoiceFilter(CharTestEnum, choice_builder=custom_choice_builder)

    class ImplicitFilterSet(EnumChoiceFilterSetMixin, drf_filters.FilterSet):
        class Meta:
            model = StringEnumeratedModel
            fields = ['enumeration']

    class ImplicitChoiceBuilderFilterSet(EnumChoiceFilterSetMixin, drf_filters.FilterSet):
        class Meta:
            model = CustomChoiceBuilderEnumeratedModel
            fields = ['enumeration']

    def setUp(self):
        for choice in CharTestEnum:
            StringEnumeratedModel.objects.create(
                enumeration=choice
            )

    def test_explicitly_declarated_field_filters_correctly(self):
        filters = {
            'enumeration': 'first'
        }
        filterset = self.ExplicitFilterSet(filters, StringEnumeratedModel.objects.all())

        self.assertEqual(filterset.qs.count(), 1)
        self.assertEqual(filterset.qs.first().enumeration, CharTestEnum.FIRST)

    def test_explicitly_declarated_field_filters_correctly_with_custom_choice_builder(self):
        filters = {
            'enumeration': 'Custom_first'
        }
        filterset = self.ExplicitChoiceBuilderFilterSet(filters, StringEnumeratedModel.objects.all())

        self.assertEqual(filterset.qs.count(), 1)
        self.assertEqual(filterset.qs.first().enumeration, CharTestEnum.FIRST)

    def test_filter_by_non_valid_choice_returns_full_queryset(self):
        filters = {
            'enumeration': 'invalid'
        }
        filterset = self.ExplicitChoiceBuilderFilterSet(filters, StringEnumeratedModel.objects.all())

        self.assertEqual(filterset.qs.count(), StringEnumeratedModel.objects.count())

    def test_implicit_filter_filters_correctly(self):
        filters = {
            'enumeration': 'first'
        }
        filterset = self.ImplicitFilterSet(filters)

        self.assertEqual(filterset.qs.count(), 1)
        self.assertEqual(filterset.qs.first().enumeration, CharTestEnum.FIRST)

    def test_implicit_filter_filters_correctly_on_field_with_custom_choice_builder(self):
        for choice in CharTestEnum:
            CustomChoiceBuilderEnumeratedModel.objects.create(enumeration=choice)

        filters = {
            'enumeration': 'Custom_first'
        }
        filterset = self.ImplicitChoiceBuilderFilterSet(filters)

        self.assertEqual(filterset.qs.count(), 1)
        self.assertEqual(filterset.qs.first().enumeration, CharTestEnum.FIRST)



from django.db import models
from django.contrib.postgres.fields import ArrayField

from django_enum_choices.fields import EnumChoiceField
from django_enum_choices.choice_builders import attribute_value

from .enumerations import CharTestEnum, CharLongValuesTestEnum, IntTestEnum


def custom_choice_builder(choice):
    return 'Custom_' + choice.value, choice.value


class IntegerEnumeratedModel(models.Model):
    enumeration = EnumChoiceField(enum_class=IntTestEnum)


class StringEnumeratedModel(models.Model):
    enumeration = EnumChoiceField(enum_class=CharTestEnum)


class NullableEnumeratedModel(models.Model):
    enumeration = EnumChoiceField(
        enum_class=CharTestEnum,
        null=True
    )


class BlankNullableEnumeratedModel(models.Model):
    enumeration = EnumChoiceField(
        enum_class=CharTestEnum,
        blank=True,
        null=True
    )


class EnumChoiceFieldWithDefaultModel(models.Model):
    enumeration = EnumChoiceField(
        CharTestEnum,
        default=CharTestEnum.FIRST
    )


class MultipleEnumeratedModel(models.Model):
    enumeration = ArrayField(
        base_field=EnumChoiceField(
            enum_class=CharTestEnum
        ),
        null=True
    )


class CustomChoiceBuilderEnumeratedModel(models.Model):
    enumeration = EnumChoiceField(
        enum_class=CharTestEnum,
        choice_builder=custom_choice_builder
    )


class AttributeChoiceBuilderEnumeratedModel(models.Model):
    enumeration = EnumChoiceField(
        enum_class=CharLongValuesTestEnum,
        choice_builder=attribute_value
    )



from django.contrib import admin
from django.conf import settings
from django.utils.translation import gettext as _

from .fields import EnumChoiceField


class EnumChoiceListFilter(admin.ChoicesFieldListFilter):
    def choices(self, changelist):
        """
        The `choices` method from `django.contrib.admin.ChoicesFieldListFilter`
        with a patch to cast lookup values from enum enumerations to their respective
        primitive values.
        """

        yield {
            'selected': self.lookup_val is None,
            'query_string': changelist.get_query_string(
                remove=[
                    self.lookup_kwarg,
                    self.lookup_kwarg_isnull
                ]
            ),
            'display': _('All')
        }
        none_title = ''
        for lookup, title in self.field.flatchoices:
            lookup = self.field.get_prep_value(lookup)

            if lookup is None:
                none_title = title
                continue
            yield {
                'selected': str(lookup) == self.lookup_val,
                'query_string': changelist.get_query_string(
                    {self.lookup_kwarg: lookup},
                    [self.lookup_kwarg_isnull]
                ),
                'display': title,
            }
        if none_title:
            yield {
                'selected': bool(self.lookup_val_isnull),
                'query_string': changelist.get_query_string(
                    {self.lookup_kwarg_isnull: 'True'},
                    [self.lookup_kwarg]
                ),
                'display': none_title,
            }

    def queryset(self, request, queryset):
        query = {
            field_name: self.field.to_enum_value(value)
            for field_name, value in self.used_parameters.items()
        }

        return queryset.filter(**query)


def register_enum_choice_list_filter():
    register_filter = getattr(
        settings,
        'DJANGO_ENUM_CHOICES_REGISTER_LIST_FILTER',
        False
    )

    if register_filter:
        admin.FieldListFilter.register(
            lambda f: isinstance(f, EnumChoiceField),
            EnumChoiceListFilter,
            take_priority=True
        )



import os
import shutil
from subprocess import check_output
import shlex

import psycopg2
from psycopg2.extras import DictCursor


class Database:
    def get_column_size(self):
        cursor = self.connection.cursor(
            cursor_factory=DictCursor
        )

        query = """
        SELECT character_maximum_length AS max_length
        FROM INFORMATION_SCHEMA.COLUMNS
        WHERE table_name = 'test_models_somemodel' and column_name = 'choices'
        """

        cursor.execute(query)

        result = cursor.fetchone()

        return result['max_length']

    def open(self):
        database_url = os.environ.get('DATABASE_URL', 'postgres:///django_enum_choices_e2e')
        self.connection = psycopg2.connect(database_url)

    def close(self):
        self.connection.close()


class Setuper:
    def __init__(self, database, test_case):
        self.cwd = os.getcwd()
        self.database = database
        self.test_case = test_case

    def get_base_project_location(self):
        return f'{self.cwd}/e2e'

    def get_new_project_location(self):
        return f'{self.cwd}/e2e_testing'

    def get_models_path(self):
        return f'{self.get_new_project_location()}/test_models/models.py'

    def call(self, command):
        result = check_output(shlex.split(command)).decode('utf-8')

        return result

    def before_setup(self):
        if not os.environ.get('CI', False):
            self.call('dropdb --if-exists django_enum_choices_e2e')
            self.call('createdb django_enum_choices_e2e')

        self.database.open()

        base_project_location = self.get_base_project_location()

        os.chdir(base_project_location)
        self.call('python manage.py migrate')

    def setup(self):
        base_project_location = self.get_base_project_location()
        new_project_location = self.get_new_project_location()

        if os.path.exists(new_project_location):
            shutil.rmtree(new_project_location)

        shutil.copytree(
            src=base_project_location,
            dst=new_project_location
        )

        models = self.get_models_path()

        with open(models, 'r') as models_file:
            models_content = models_file.read()

        models_content = models_content.replace(f' #{self.test_case}', '')

        with open(models, 'w') as models_file:
            models_file.write(models_content)

    def after_setup(self):
        new_project_location = self.get_new_project_location()

        if os.path.exists(new_project_location):
            shutil.rmtree(new_project_location)

        os.chdir(self.cwd)
        self.database.close()


def test_case_1():
    """
    If we add new choice, which changes the max length, we should have a migration,
    plus the column in postgres should be affected.
    """
    print('Running test case 1')

    database = Database()

    setuper = Setuper(database, 'TEST_1')

    setuper.before_setup()
    assert database.get_column_size() == 3, 'Initial max length should be 3'

    setuper.setup()

    new_project_location = setuper.get_new_project_location()

    os.chdir(new_project_location)

    result = setuper.call('python manage.py makemigrations')

    assert 'No changes detected' not in result, 'There should be new migrations'

    print(result)

    print(setuper.call('python manage.py migrate'))

    assert database.get_column_size() == 4, 'Migration should increase the max length to 4'

    setuper.after_setup()


def test_case_2():
    """
    If we add new choice, but within the existing max length, we should have a new migration.
    """
    print('Running test case 2')

    database = Database()

    setuper = Setuper(database, 'TEST_2')
    setuper.before_setup()
    setuper.setup()

    new_project_location = setuper.get_new_project_location()

    os.chdir(new_project_location)

    result = setuper.call('python manage.py makemigrations')

    assert 'No changes detected' not in result, 'There should be new migrations'

    print(result)

    print(setuper.call('python manage.py migrate'))

    setuper.after_setup()


test_case_1()
test_case_2()



from enum import Enum

from django.db import models

from django_enum_choices.fields import EnumChoiceField


class ChoicesA(Enum):
    A = 'a'
    B = 'aa'
    C = 'aaa'
    #TEST_1 D = 'aaaa'


class ChoicesB(Enum):
    A = 'a'
    #TEST_2 B = 'b'


class SomeModel(models.Model):
    choices = EnumChoiceField(ChoicesA)

    other_choices = EnumChoiceField(ChoicesB)



from .enumerations import MyEnum
from .models import MyModel, MyModelMultiple
from .serializers import (
    MySerializer,
    MyModelSerializer,
    ImplicitMyModelSerializer,
    MultipleMySerializer,
    ImplicitMultipleMyModelSerializer,
    CustomChoiceBuilderSerializer
)
from .forms import StandardEnumForm, ModelEnumForm, CustomChoiceBuilderEnumForm
from .filters import ExplicitFilterSet, ExplicitCustomChoiceBuilderFilterSet, ImplicitFilterSet


# Object Creation
def create_instance():
    return MyModel.objects.create(enumerated_field=MyEnum.A)


# Overriding field value
def update_instance():
    instance = create_instance()
    instance.enumerated_field = MyEnum.B

    instance.save()

    return instance


# Object creation with multiple field
def create_instance_with_multiple_field():
    return MyModelMultiple.objects.create(enumerated_field=[MyEnum.A, MyEnum.B])


# Overriding multiple field value
def update_instance_with_multiple_field():
    instance = create_instance_with_multiple_field()
    instance.enumerated_field = [MyEnum.B]

    instance.save()

    return instance


# QuerySet filtering
def filter_queryset():
    return MyModel.objects.filter(enumerated_field=MyEnum.A)


# Serializer usage
def serialize_value():
    serializer = MySerializer({
        'enumerated_field': MyEnum.A
    })

    return serializer.data


def deserialize_value():
    serializer = MySerializer(data={
        'enumerated_field': 'a'
    })
    serializer.is_valid()

    return serializer.validated_data


# Explicit ModelSerializer usage
def serialize_model_from_explicit_serializer():
    instance = create_instance()
    serializer = MyModelSerializer(instance)

    return serializer.data


def create_model_from_explicit_serializer():
    serializer = MyModelSerializer(data={
        'enumerated_field': 'a'
    })
    serializer.is_valid()

    return serializer.save()


# Implicit ModelSerializer usage
def serialize_model_from_implicit_serializer():
    instance = create_instance()
    serializer = ImplicitMyModelSerializer(instance)

    return serializer.data


def create_model_from_implicit_serializer():
    serializer = ImplicitMyModelSerializer(data={
        'enumerated_field': 'a'
    })
    serializer.is_valid()

    return serializer.save()


# Multiple Standard Serializer Usage
def serialize_multiple_value():
    serializer = MultipleMySerializer({
        'enumerated_field': [MyEnum.A, MyEnum.B]
    })

    return serializer.data


def deserialize_multiple_value():
    serializer = MultipleMySerializer(data={
        'enumerated_field': ['a', 'b']
    })
    serializer.is_valid()

    return serializer.validated_data


# Implicit Multiple ModelSerializer usage
def serialize_model_from_multiple_field_serializer():
    instance = create_instance_with_multiple_field()
    serializer = ImplicitMultipleMyModelSerializer(instance)

    return serializer.data


def create_model_from_multiple_field_serializer():
    serializer = ImplicitMultipleMyModelSerializer(data={
        'enumerated_field': ['a', 'b']
    })
    serializer.is_valid()

    return serializer.save()


def serialize_with_custom_choice_builder():
    serializer = CustomChoiceBuilderSerializer({
        'enumerated_field': MyEnum.A
    })

    return serializer.data


def get_value_from_standard_form():
    form = StandardEnumForm({
        'enumerated_field': 'a'
    })

    form.is_valid()

    return form.cleaned_data


def create_instance_from_model_form():
    form = ModelEnumForm({
        'enumerated_field': 'a'
    })

    form.is_valid()

    return form.save(commit=True)


def get_value_from_form_with_custom_choice_builder_field():
    form = CustomChoiceBuilderEnumForm({
        'enumerated_field': 'Custom_a'
    })

    form.is_valid()

    return form.cleaned_data


def filter_with_explicit_field():
    for choice in MyEnum:
        MyModel.objects.create(enumerated_field=choice)

    filters = {
        'enumerated_field': 'a'
    }
    filterset = ExplicitFilterSet(filters, MyModel.objects.all())

    return filterset.qs.values_list('enumerated_field', flat=True)


def filter_with_explicit_field_with_custom_choice_builder():
    for choice in MyEnum:
        MyModel.objects.create(enumerated_field=choice)

    filters = {
        'enumerated_field': 'Custom_a'
    }
    filterset = ExplicitCustomChoiceBuilderFilterSet(filters, MyModel.objects.all())

    return filterset.qs.values_list('enumerated_field', flat=True)


def filter_with_implicit_field():
    for choice in MyEnum:
        MyModel.objects.create(enumerated_field=choice)

    filters = {
        'enumerated_field': 'a'
    }
    filterset = ImplicitFilterSet(filters)

    return filterset.qs.values_list('enumerated_field', flat=True)



import django_filters as filters

from .fields import EnumChoiceField
from .forms import EnumChoiceField as EnumChoiceFormField
from .choice_builders import value_value


class EnumChoiceFilter(filters.ChoiceFilter):
    field_class = EnumChoiceFormField

    def __init__(self, enum_class, choice_builder=value_value, *args, **kwargs):
        super().__init__(
            enum_class=enum_class,
            choice_builder=choice_builder,
            *args,
            **kwargs
        )


class EnumChoiceFilterSetMixin:
    """
    `django-filter` has specific logic for handling fields with `choices`.
    We need to override `filter_for_lookup` to return an `EnumChoiceFilter`
    before `django-filter` returns a `ChoiceFilter` as the `filter_class`
    for the `EnumChoiceField` instances in the model.
    """

    @classmethod
    def filter_for_lookup(cls, field, lookup_type):
        if isinstance(field, EnumChoiceField):
            return EnumChoiceFilter, {
                'enum_class': field.enum_class,
                'choice_builder': field.choice_builder
            }

        return super().filter_for_lookup(field, lookup_type)



from django.test import TestCase
from django import forms

from django_enum_choices.forms import EnumChoiceField

from .testapp.enumerations import CharTestEnum
from .testapp.models import StringEnumeratedModel, CustomChoiceBuilderEnumeratedModel


def custom_choice_builder(choice):
    return 'Custom_' + choice.value, choice.value


class FormIntegrationTests(TestCase):
    class StandardEnumForm(forms.Form):
        enumerated_field = EnumChoiceField(CharTestEnum)

    class CustomChoiceBuilderEnumForm(forms.Form):
        enumerated_field = EnumChoiceField(
            CharTestEnum,
            choice_builder=custom_choice_builder
        )

    def test_value_is_cleaned_successfully_when_value_is_valid(self):
        form = self.StandardEnumForm({
            'enumerated_field': 'first'
        })

        self.assertTrue(form.is_valid())
        self.assertEqual(
            CharTestEnum.FIRST,
            form.cleaned_data['enumerated_field']
        )

    def test_form_is_not_valid_when_value_is_not_in_enum_class(self):
        form = self.StandardEnumForm({
            'enumerated_field': 'not_valid'
        })

        expected_error = EnumChoiceField.default_error_messages['invalid_choice'] % {'value': 'not_valid'}

        self.assertFalse(form.is_valid())
        self.assertIn(expected_error, form.errors.get('enumerated_field', []))

    def test_form_is_not_valid_when_value_is_none_and_field_is_required(self):
        form = self.StandardEnumForm({
            'enumerated_field': None
        })

        expected_error = 'This field is required.'

        self.assertFalse(form.is_valid())
        self.assertIn(expected_error, form.errors.get('enumerated_field', []))

    def test_form_is_valid_when_value_is_none_and_field_is_not_required(self):
        class NonRequiredForm(forms.Form):
            enumerated_field = EnumChoiceField(CharTestEnum, required=False)

        form = NonRequiredForm({
            'enumerated_field': None
        })

        self.assertTrue(form.is_valid())
        self.assertEqual(None, form.cleaned_data['enumerated_field'])

    def test_form_is_valid_when_value_is_valid_and_field_uses_custom_choice_builder(self):
        form = self.CustomChoiceBuilderEnumForm({
            'enumerated_field': 'Custom_first'
        })

        self.assertTrue(form.is_valid())
        self.assertEqual(
            CharTestEnum.FIRST,
            form.cleaned_data['enumerated_field']
        )

    def test_form_is_not_valid_when_value_is_invalid_and_field_uses_custom_choice_builder(self):
        form = self.CustomChoiceBuilderEnumForm({
            'enumerated_field': 'first'
        })

        expected_error = EnumChoiceField.default_error_messages['invalid_choice'] % {'value': 'first'}

        self.assertFalse(form.is_valid())
        self.assertIn(expected_error, form.errors['enumerated_field'])


class ModelFormIntegrationTests(TestCase):
    class StandardEnumForm(forms.ModelForm):
        class Meta:
            model = StringEnumeratedModel
            fields = ('enumeration', )

    class CustomChoiceBuilderEnumForm(forms.ModelForm):
        class Meta:
            model = CustomChoiceBuilderEnumeratedModel
            fields = ('enumeration', )

    def test_form_is_valid_when_value_is_valid(self):
        form = self.StandardEnumForm({
            'enumeration': 'first'
        })

        self.assertTrue(form.is_valid())
        self.assertEqual(CharTestEnum.FIRST, form.cleaned_data['enumeration'])

    def test_form_is_invalid_when_value_is_not_from_choices(self):
        form = self.StandardEnumForm({
            'enumeration': 'not_valid'
        })

        expected_error = EnumChoiceField.default_error_messages['invalid_choice'] % {'value': 'not_valid'}

        self.assertFalse(form.is_valid())
        self.assertIn(expected_error, form.errors.get('enumeration', []))

    def test_form_is_valid_when_value_is_valid_and_form_uses_custom_choice_builder(self):
        form = self.CustomChoiceBuilderEnumForm({
            'enumeration': 'Custom_first'
        })

        self.assertTrue(form.is_valid())
        self.assertEqual(CharTestEnum.FIRST, form.cleaned_data['enumeration'])

    def test_form_is_invalid_when_value_is_not_from_choices_and_form_uses_custom_choice_builder(self):
        form = self.CustomChoiceBuilderEnumForm({
            'enumeration': 'first'
        })

        self.assertFalse(form.is_valid())

        expected_error = EnumChoiceField.default_error_messages['invalid_choice'] % {'value': 'first'}
        self.assertIn(expected_error, form.errors.get('enumeration', []))

    def test_saving_model_form_creates_instance(self):
        form = self.CustomChoiceBuilderEnumForm({
            'enumeration': 'Custom_first'
        })

        current_instance_count = CustomChoiceBuilderEnumeratedModel.objects.count()

        self.assertTrue(form.is_valid())

        instance = form.save(commit=True)

        self.assertEqual(
            current_instance_count + 1,
            CustomChoiceBuilderEnumeratedModel.objects.count()
        )
        self.assertEqual(
            CharTestEnum.FIRST,
            instance.enumeration
        )



from django.test import TestCase
from django.core import serializers
from django.core.exceptions import ValidationError

from django_enum_choices.fields import EnumChoiceField

from .testapp.enumerations import CharTestEnum, CharLongValuesTestEnum, IntTestEnum
from .testapp.models import (
    IntegerEnumeratedModel,
    StringEnumeratedModel,
    NullableEnumeratedModel,
    BlankNullableEnumeratedModel,
    EnumChoiceFieldWithDefaultModel,
    AttributeChoiceBuilderEnumeratedModel
)


class ModelIntegrationTests(TestCase):
    def test_can_create_object_with_char_base(self):
        instance = StringEnumeratedModel.objects.create(
            enumeration=CharTestEnum.FIRST
        )
        instance.refresh_from_db()

        self.assertEqual(instance.enumeration, CharTestEnum.FIRST)

    def test_can_assign_enumeration_with_char_base(self):
        instance = StringEnumeratedModel.objects.create(
            enumeration=CharTestEnum.FIRST
        )
        instance.refresh_from_db()

        instance.enumeration = CharTestEnum.SECOND
        instance.save()
        instance.refresh_from_db()

        self.assertEqual(instance.enumeration, CharTestEnum.SECOND)

    def test_can_filter_by_enumeration_with_char_base(self):
        first = StringEnumeratedModel.objects.create(
            enumeration=CharTestEnum.FIRST
        )
        second = StringEnumeratedModel.objects.create(
            enumeration=CharTestEnum.SECOND
        )

        first_qs = StringEnumeratedModel.objects.filter(enumeration=CharTestEnum.FIRST)
        second_qs = StringEnumeratedModel.objects.filter(enumeration=CharTestEnum.SECOND)

        self.assertIn(first, first_qs)
        self.assertNotIn(second, first_qs)

        self.assertIn(second, second_qs)
        self.assertNotIn(first, second_qs)

    def test_can_create_object_with_int_base(self):
        instance = IntegerEnumeratedModel.objects.create(
            enumeration=IntTestEnum.FIRST
        )
        instance.refresh_from_db()

        self.assertEqual(instance.enumeration, IntTestEnum.FIRST)

    def test_can_assign_enumeration_with_int_base(self):
        instance = IntegerEnumeratedModel.objects.create(
            enumeration=IntTestEnum.FIRST
        )
        instance.refresh_from_db()

        instance.enumeration = IntTestEnum.SECOND
        instance.save()
        instance.refresh_from_db()

        self.assertEqual(instance.enumeration, IntTestEnum.SECOND)

    def test_can_filter_by_enumeration_with_int_base(self):
        first = IntegerEnumeratedModel.objects.create(
            enumeration=IntTestEnum.FIRST
        )
        second = IntegerEnumeratedModel.objects.create(
            enumeration=IntTestEnum.SECOND
        )

        first_qs = IntegerEnumeratedModel.objects.filter(enumeration=IntTestEnum.FIRST)
        second_qs = IntegerEnumeratedModel.objects.filter(enumeration=IntTestEnum.SECOND)

        self.assertIn(first, first_qs)
        self.assertNotIn(second, first_qs)

        self.assertIn(second, second_qs)
        self.assertNotIn(first, second_qs)

    def test_serialization(self):
        IntegerEnumeratedModel.objects.create(
            enumeration=IntTestEnum.FIRST
        )

        data = serializers.serialize('json', IntegerEnumeratedModel.objects.all())

        expected = '[{"model": "testapp.integerenumeratedmodel", "pk": 1, "fields": {"enumeration": "1"}}]'

        self.assertEqual(expected, data)

    def test_deserialization(self):
        instance = IntegerEnumeratedModel.objects.create(
            enumeration=IntTestEnum.FIRST
        )

        data = serializers.serialize('json', IntegerEnumeratedModel.objects.all())
        objects = list(serializers.deserialize('json', data))

        self.assertEqual(1, len(objects))

        deserialized_instance = objects[0]

        self.assertEqual(instance, deserialized_instance.object)

    def test_object_with_nullable_field_can_be_created(self):
        instance = NullableEnumeratedModel()

        self.assertIsNone(instance.enumeration)

    def test_nullable_field_can_be_set_to_none(self):
        instance = NullableEnumeratedModel(
            enumeration=CharTestEnum.FIRST
        )

        instance.enumeration = None
        instance.save()
        instance.refresh_from_db()

        self.assertIsNone(instance.enumeration)

    def test_non_blank_field_raises_error_on_clean(self):
        instance = NullableEnumeratedModel()

        with self.assertRaisesMessage(
            ValidationError,
            str(EnumChoiceField.default_error_messages['blank'])
        ):
            instance.full_clean()

    def test_blank_field_does_not_raise_error_on_clean(self):
        instance = BlankNullableEnumeratedModel()

        instance.full_clean()

        self.assertIsNone(instance.enumeration)

    def test_non_enum_value_raises_error_on_clean(self):
        instance = StringEnumeratedModel.objects.create(
            enumeration=CharTestEnum.FIRST
        )

        instance.enumeration = "foo"

        with self.assertRaises(
            ValidationError
        ):
            instance.full_clean()

    def test_enum_value_from_enum_class_does_not_raise_error_on_clean(self):
        instance = StringEnumeratedModel(enumeration=CharTestEnum.FIRST)
        instance.full_clean()
        instance.save()

        self.assertIsNotNone(instance)

    def test_attribute_choice_builder_validates_enum_field_name_length(self):
        instance = AttributeChoiceBuilderEnumeratedModel(enumeration=CharLongValuesTestEnum.FIRST)
        instance.full_clean()
        instance.save()

        self.assertIsNotNone(instance)

    def test_default_value_is_used(self):
        instance = EnumChoiceFieldWithDefaultModel.objects.create()

        self.assertEqual(
            EnumChoiceFieldWithDefaultModel._meta.get_field('enumeration').default,
            instance.enumeration
        )



from django.test import TestCase, RequestFactory, override_settings
from django.contrib import admin
from django.contrib.auth.models import User

from django_enum_choices.admin import EnumChoiceListFilter, register_enum_choice_list_filter

from .testapp.models import StringEnumeratedModel
from .testapp.enumerations import CharTestEnum


class StringEnumAdmin(admin.ModelAdmin):
    list_filter = ('enumeration', )


class EnumChoiceListFilterTests(TestCase):
    request_factory = RequestFactory()

    def setUp(self):
        self.user = User.objects.create_superuser('user', 'user@example.com', 'password')

    # Django 1.11 compatibillity
    def get_changelist_instance(self, request, model, modeladmin):
        changelist_args = [
            request, model, modeladmin.list_display,
            modeladmin.list_display_links, modeladmin.list_filter,
            modeladmin.date_hierarchy, modeladmin.search_fields,
            modeladmin.list_select_related, modeladmin.list_per_page,
            modeladmin.list_max_show_all, modeladmin.list_editable, modeladmin,
        ]

        try:
            changelist = modeladmin.get_changelist(request)(*changelist_args)
        except TypeError:
            # Django < 2 does not accept `sortable_by` in `ChangeList`
            changelist_args.append(modeladmin.sortable_by)
            changelist = modeladmin.get_changelist(request)(*changelist_args)

        return changelist

    @override_settings(DJANGO_ENUM_CHOICES_REGISTER_LIST_FILTER=True)
    def test_list_filter_instance_is_enumchoicelistfilter_when_registered(self):
        register_enum_choice_list_filter()

        modeladmin = StringEnumAdmin(
            StringEnumeratedModel,
            admin.site
        )
        request = self.request_factory.get('/', {})
        request.user = self.user

        changelist = self.get_changelist_instance(request, StringEnumeratedModel, modeladmin)
        filterspec = changelist.get_filters(request)[0][0]

        self.assertIsInstance(filterspec, EnumChoiceListFilter)

    @override_settings(DJANGO_ENUM_CHOICES_REGISTER_LIST_FILTER=True)
    def test_list_filter_lookup_kwarg_is_correct(self):
        register_enum_choice_list_filter()

        modeladmin = StringEnumAdmin(
            StringEnumeratedModel,
            admin.site
        )
        request = self.request_factory.get('/', {})
        request.user = self.user

        changelist = self.get_changelist_instance(request, StringEnumeratedModel, modeladmin)
        filterspec = changelist.get_filters(request)[0][0]

        self.assertEqual(filterspec.lookup_kwarg, 'enumeration__exact')

    @override_settings(DJANGO_ENUM_CHOICES_REGISTER_LIST_FILTER=True)
    def test_list_filter_queryset_filters_objects_correctly(self):
        StringEnumeratedModel.objects.create(enumeration=CharTestEnum.FIRST)
        StringEnumeratedModel.objects.create(enumeration=CharTestEnum.SECOND)
        StringEnumeratedModel.objects.create(enumeration=CharTestEnum.THIRD)

        modeladmin = StringEnumAdmin(
            StringEnumeratedModel,
            admin.site
        )

        for enumeration in CharTestEnum:
            request = self.request_factory.get('/', {'enumeration__exact': enumeration.value})
            request.user = self.user

            changelist = self.get_changelist_instance(request, StringEnumeratedModel, modeladmin)

            self.assertEqual(changelist.queryset.count(), 1)



from typing import Callable

from django.core.validators import MaxLengthValidator


class EnumValueMaxLengthValidator(MaxLengthValidator):
    """
    When called from the field's `run_validators`, `MaxLengthValidator`
    attempts to return `len(value)` when value is an enumeration
    instance, which raises an error
    """
    def __init__(self, value_builder: Callable,  *args, **kwargs):
        self.value_builder = value_builder

        super().__init__(*args, **kwargs)

    def clean(self, x):
        value = self.value_builder(x)

        return len(value)



from django import forms

from .choice_builders import value_value
from .utils import as_choice_builder, value_from_built_choice, build_enum_choices


class EnumChoiceField(forms.ChoiceField):
    def __init__(self, enum_class, choice_builder=value_value, **kwargs):
        self.enum_class = enum_class
        self.choice_builder = as_choice_builder(choice_builder)

        # Special case where choices are autogenerated from `fields.EnumChoiceField.formfield`
        # The `choices` argument is not to be used outside of `fields.EnumChoiceField.formfield`
        if 'choices' not in kwargs:
            kwargs['choices'] = self.build_choices()

        super().__init__(**kwargs)

    def build_choices(self):
        return build_enum_choices(
            self.enum_class,
            self.choice_builder
        )

    def _enum_from_input_value(self, value):
        for choice in self.enum_class:
            if value_from_built_choice(self.choice_builder(choice)) == value:
                return choice

    def to_python(self, value):
        if value is None:
            return

        return self._enum_from_input_value(value) or value

    def prepare_value(self, value):
        if not value or isinstance(value, self.enum_class):
            return value_from_built_choice(
                self.choice_builder(value)
            )

        return value

    def valid_value(self, value):
        return value in self.enum_class



from enum import Enum

from django.test import TestCase
from django.core.exceptions import ValidationError
from django.contrib.admin.utils import display_for_field

from django_enum_choices.fields import EnumChoiceField
from django_enum_choices.exceptions import EnumChoiceFieldException
from django_enum_choices.choice_builders import value_value
from django_enum_choices.forms import EnumChoiceField as EnumChoiceFormField

from .testapp.enumerations import CharTestEnum, IntTestEnum


class EnumChoiceFieldTests(TestCase):
    def test_field_initializes_string_choice_values(self):
        field = EnumChoiceField(enum_class=CharTestEnum)

        self.assertEqual(
            field.enum_class,
            CharTestEnum
        )

        expected_choices = [
            ('first', 'first'),
            ('second', 'second'),
            ('third', 'third'),
        ]

        self.assertEqual(
            expected_choices,
            field.choices,
        )

    def test_field_initializes_int_choice_values_by_stringifying_them(self):
        field = EnumChoiceField(enum_class=IntTestEnum)

        self.assertEqual(
            field.enum_class,
            IntTestEnum
        )

        expected_choices = [
            ('1', '1'),
            ('2', '2'),
            ('3', '3'),
        ]

        self.assertEqual(
            expected_choices,
            field.choices,
        )

    def test_field_initializes_arbitrary_object_values_by_stringifying_them(self):
        class Foo:
            def __str__(self):
                return 'foo'

        class TestEnum(Enum):
            FOO = Foo()

        field = EnumChoiceField(enum_class=TestEnum)

        expected_choices = [
            ('foo', 'foo')
        ]

        self.assertEqual(
            expected_choices,
            field.choices,
        )

    def test_max_length_is_calculated_from_the_longest_element(self):

        class TestEnum(Enum):
            FOO = 'foo'
            BAR = 'A' * 100

        field = EnumChoiceField(enum_class=TestEnum)

        self.assertEqual(100, field.max_length)

    def test_field_raises_exception_when_enum_class_is_not_enumeration(self):
        class FailingEnum:
            FOO = 'foo'
            BAR = 'bar'

        with self.assertRaisesMessage(
            EnumChoiceFieldException,
            '`enum_class` argument must be a child of `Enum`'
        ):
            EnumChoiceField(enum_class=FailingEnum)

    def test_get_prep_value_returns_primitive_value_when_base_is_integer(self):
        instance = EnumChoiceField(enum_class=IntTestEnum)

        result = instance.get_prep_value(IntTestEnum.FIRST)

        self.assertEqual(result, '1')

    def test_get_prep_value_returns_primitive_value_when_base_is_string(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        result = instance.get_prep_value(CharTestEnum.FIRST)

        self.assertEqual(result, 'first')

    def test_from_db_value_returns_none_when_value_is_none(self):
        instance = EnumChoiceField(enum_class=IntTestEnum)

        result = instance.from_db_value(None, None, None)

        self.assertIsNone(result)

    def test_from_db_value_returns_enum_value_when_base_is_integer(self):
        instance = EnumChoiceField(enum_class=IntTestEnum)

        result = instance.from_db_value('1', None, None)

        self.assertEqual(result, IntTestEnum.FIRST)

    def test_from_db_value_returns_enum_value_when_base_is_string(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        result = instance.from_db_value('first', None, None)

        self.assertEqual(result, CharTestEnum.FIRST)

    def test_from_db_value_raises_exception_when_int_value_not_contained_in_enum_class(self):
        instance = EnumChoiceField(enum_class=IntTestEnum)

        with self.assertRaises(ValidationError):
            instance.from_db_value(7, None, None)

    def test_deconstruct_behaves_as_expected(self):
        """
        Idea taken from:
        https://docs.djangoproject.com/en/2.2/howto/custom-model-fields/#field-deconstruction
        """
        instance = EnumChoiceField(enum_class=IntTestEnum)
        name, path, args, kwargs = instance.deconstruct()

        new_instance = EnumChoiceField(*args, **kwargs)

        self.assertEqual(instance.enum_class, new_instance.enum_class)
        self.assertEqual(instance.choices, new_instance.choices)
        self.assertEqual(instance.max_length, new_instance.max_length)
        self.assertEqual(instance._original_choice_builder, new_instance._original_choice_builder)

    def test_choice_builder_is_used_when_callable(self):
        class TestEnum(Enum):
            A = 1
            B = 2

        def choice_builder(enum_instance):
            if enum_instance == TestEnum.A:
                return 1, 'A'

            if enum_instance == TestEnum.B:
                return 2, 'B'

        instance = EnumChoiceField(enum_class=TestEnum, choice_builder=choice_builder)

        expected_choices = [
            ('1', 'A'),
            ('2', 'B')
        ]

        self.assertEqual(expected_choices, instance.choices)

    def test_to_python_returns_enum_when_called_with_enum_value(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        result = instance.to_python(CharTestEnum.FIRST)

        self.assertEqual(CharTestEnum.FIRST, result)

    def test_to_python_returns_enum_when_called_with_primitive_value(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        result = instance.to_python('first')

        self.assertEqual(CharTestEnum.FIRST, result)

    def test_to_python_returns_none_when_called_with_none(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        result = instance.to_python(None)

        self.assertIsNone(result)

    def test_to_python_raises_exception_when_called_with_value_outside_enum_class(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        with self.assertRaises(
            ValidationError
        ):
            instance.to_python('NOT_EXISTING')

    def test_flatchoices_returns_enumerations_as_choice_keys(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        result = instance.flatchoices

        for choice, _ in result:
            self.assertIsInstance(choice, CharTestEnum)

    def test_flatchoices_returns_readable_value_as_choice_value_when_autogenerated(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        result = instance.flatchoices

        for choice, readable in result:
            self.assertEqual(
                CharTestEnum(choice).value,
                readable
            )

    def test_flatchoices_returns_readable_value_as_choice_value_when_choice_builder_is_redefined(self):
        class TestEnum(Enum):
            FOO = 'foo'
            BAR = 'bar'

        def choice_builder(choice):
            return choice.value, choice.value.upper()

        instance = EnumChoiceField(enum_class=TestEnum, choice_builder=choice_builder)

        result = instance.flatchoices

        for choice, readable in result:
            self.assertEqual(
                choice_builder(TestEnum(choice))[1],
                readable
            )

    def test_display_for_field_returns_readable_value_when_autogenerated(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        result = display_for_field(CharTestEnum.FIRST, instance, None)

        self.assertEqual(CharTestEnum.FIRST.value, result)

    def test_display_for_field_returns_readable_value_when_choice_builder_is_redefined(self):
        class TestEnum(Enum):
            FOO = 'foo'
            BAR = 'bar'

        def choice_builder(choice):
            return choice.value, choice.value.upper()

        instance = EnumChoiceField(enum_class=TestEnum, choice_builder=choice_builder)

        result = display_for_field(TestEnum.FOO, instance, None)

        self.assertEqual(choice_builder(TestEnum.FOO)[1], result)

    def test_display_for_field_returns_empty_display_when_value_is_none(self):
        EMPTY_DISPLAY = 'EMPTY'

        instance = EnumChoiceField(enum_class=CharTestEnum)

        result = display_for_field(None, instance, EMPTY_DISPLAY)

        self.assertEqual(EMPTY_DISPLAY, result)

    def test_validate_raises_error_when_field_is_not_null_and_value_is_none(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        with self.assertRaisesMessage(
            ValidationError,
            str(instance.error_messages['null'])
        ):
            instance.validate(None)

    def test_validate_raises_error_when_field_is_not_blank_and_value_is_empty(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        with self.assertRaisesMessage(
            ValidationError,
            str(instance.error_messages['blank'])
        ):
            instance.validate('')

    def test_validate_raises_error_when_value_is_not_from_enum_class(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        expected = instance.error_messages['invalid_choice'] % {'value': 'foo'}

        with self.assertRaisesMessage(
            ValidationError,
            expected
        ):
            instance.validate('foo')

    def test_get_choice_builder_raises_exception_when_choice_builder_is_not_callable(self):
        class TestEnum(Enum):
            A = 1
            B = 2

        choice_builder = 'choice_builder'

        with self.assertRaisesMessage(
            EnumChoiceFieldException,
            '`TestEnum.choice_builder` must be a callable'
        ):
            instance = EnumChoiceField(enum_class=TestEnum)

            instance._get_choice_builder(choice_builder)

    def test_value_value_is_used_when_choice_builder_is_not_provided(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        self.assertEqual(instance._original_choice_builder, value_value)

    def test_custom_choice_builder_is_used_when_provided_and_callable(self):
        def choice_builder(choice):
            return choice.name, choice.value

        instance = EnumChoiceField(
            enum_class=CharTestEnum,
            choice_builder=choice_builder
        )

        self.assertEqual(
            instance._original_choice_builder,
            choice_builder
        )

    def test_build_choices_raises_exception_when_not_all_values_are_strings(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)
        instance.choice_builder = lambda x: (1, 1)

        with self.assertRaisesMessage(
            EnumChoiceFieldException,
            'Received type {} on key inside choice: (1, 1).\n'.format(int) +
            'All choices generated from {} must be strings.'.format(CharTestEnum)
        ):
            instance.build_choices()

    def test_get_choices_returns_choices_in_correct_format(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        result = instance.get_choices()

        self.assertEqual(len(result), 4)
        self.assertIn(instance.choice_builder(CharTestEnum.FIRST), result)
        self.assertIn(instance.choice_builder(CharTestEnum.SECOND), result)
        self.assertIn(instance.choice_builder(CharTestEnum.THIRD), result)

    def test_formfield_returns_enum_choice_form_field_instance(self):
        instance = EnumChoiceField(enum_class=CharTestEnum)

        result = instance.formfield()

        self.assertIsInstance(result, EnumChoiceFormField)



from django.db import models
from django.contrib.postgres.fields import ArrayField

from django_enum_choices.fields import EnumChoiceField

from .enumerations import MyEnum


# Model, containing a field with enumerations as choices
class MyModel(models.Model):
    enumerated_field = EnumChoiceField(MyEnum)


# Model, containing a field with a array of enumerations (PostgreSQL specific)
class MyModelMultiple(models.Model):
    enumerated_field = ArrayField(
        base_field=EnumChoiceField(MyEnum)
    )


def choice_builder(choice):
    return choice.value, choice.value.upper()


class CustomReadableValueEnumModel(models.Model):
    enumerated_field = EnumChoiceField(
        MyEnum,
        choice_builder=choice_builder
    )



from enum import Enum, auto


# Standard enum
class MyEnum(Enum):
    A = 'a'
    B = 'b'


# Enum with custom objects as values
class Value:
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value


class CustomObjectEnum(Enum):
    A = Value(1)
    B = Value('B')


# Enum with autogenerated values
class AutoEnum(Enum):
    A = auto()  # 1
    B = auto()  # 2


class CustomAutoEnumValueGenerator(Enum):
    def _generate_next_value_(name, start, count, last_values):
        return {
            'A': 'foo',
            'B': 'bar'
        }[name]


class CustomAutoEnum(CustomAutoEnumValueGenerator):
    A = auto()
    B = auto()



from enum import Enum
from typing import Tuple, Type

from django.db.models import CharField
from django.core.exceptions import ValidationError
from django.core.validators import MaxLengthValidator
from django.utils.translation import gettext as _
from django.utils.text import capfirst

from .exceptions import EnumChoiceFieldException
from .validators import EnumValueMaxLengthValidator
from .choice_builders import value_value
from .utils import as_choice_builder, value_from_built_choice, build_enum_choices
from .forms import EnumChoiceField as EnumChoiceFormField


class EnumChoiceField(CharField):
    description = _('EnumChoiceField for %(enum_class)')

    def __init__(self, enum_class: Type[Enum], choice_builder=value_value, **kwargs):
        if not issubclass(enum_class, Enum):
            raise EnumChoiceFieldException(
                _('`enum_class` argument must be a child of `Enum`')
            )

        self.enum_class = enum_class
        self.choice_builder = self._get_choice_builder(choice_builder)

        # Saving original for proper deconstruction
        self._original_choice_builder = choice_builder

        built_choices = self.build_choices()

        # `choices` is passed to `__init__` when migrations are generated
        # `choices` should not be passed when normally initializing the field
        self._passed_choices = kwargs.get(
            'choices',
            built_choices
        )  # Saved for deconstruction

        kwargs['choices'] = built_choices

        calculated_max_length = self._calculate_max_length(**kwargs)

        kwargs.setdefault('max_length', calculated_max_length)

        super().__init__(**kwargs)

        # Removing `MaxLengthValidator` instances and adding
        # an `EnumValueMaxLengthValidator` instance
        self.validators = [
            validator for validator in self.validators
            if not isinstance(validator, MaxLengthValidator)
        ]

        self.validators.append(
            EnumValueMaxLengthValidator(
                value_builder=self.get_prep_value,
                limit_value=kwargs['max_length']
            )
        )

    def _get_choice_builder(self, choice_builder):
        if not callable(choice_builder):
            raise EnumChoiceFieldException(
                _('`{}.choice_builder` must be a callable.'.format(
                    self.enum_class.__name__
                ))
            )

        return as_choice_builder(choice_builder)

    def build_choices(self) -> Tuple[Tuple[str]]:
        return build_enum_choices(
            self.enum_class,
            self.choice_builder
        )

    def _calculate_max_length(self, **kwargs) -> int:
        max_choice_length = max(len(choice) for choice, _ in kwargs['choices'])

        return max_choice_length

    def to_enum_value(self, value):
        if value is None:
            return

        for choice in self.enum_class:
            # Check if the value from the built choice matches the passed one
            if value_from_built_choice(self.choice_builder(choice)) == value:
                return choice

        raise ValidationError(
            _('Value {} not found in {}'.format(value, self.enum_class))
        )

    def get_prep_value(self, value):
        return value_from_built_choice(
            self.choice_builder(value)
        )

    def from_db_value(self, value, expression, connection, *args):
        # Accepting `*args` because Django 1.11 calls with an extra
        # `context` argument

        return self.to_enum_value(value)

    def to_python(self, value):
        if isinstance(value, self.enum_class):
            return value

        return self.to_enum_value(value)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()

        kwargs['choices'] = self._passed_choices

        if self.enum_class:
            kwargs['enum_class'] = self.enum_class

        if self.choice_builder:
            kwargs['choice_builder'] = self._original_choice_builder

        return name, path, args, kwargs

    def validate(self, value, *args, **kwargs):
        """
        Runs standard `django.db.models.Field` validation
        with different logic for choices validation
        """

        if not self.editable:
            return

        if value is None and not self.null:
            raise ValidationError(self.error_messages['null'], code='null')

        if not self.blank and value in self.empty_values:
            raise ValidationError(self.error_messages['blank'], code='blank')

        if value not in self.enum_class:
            raise ValidationError(
                self.error_messages['invalid_choice'],
                code='invalid_choice',
                params={'value': value}
            )

    def value_to_string(self, obj):
        value = self.value_from_object(obj)
        return self.get_prep_value(value)

    @property
    def flatchoices(self):
        """
        Django admin uses `flatchoices` to generate a value under the
        fields column in their list display. By default it calculates
        `flatchoices` as a Tuple[Tuple[str]],
        I.E: `(('choice1', 'choice1'), ('choice2', 'choice2'))
        It accesses the readable value by using the actual value
        which is an enumeration instance in our case.
        Since that does not match inside the original `flatchoices`
        it sets the display value to `-`.
        """

        flatchoices = super()._get_flatchoices()

        return [
            (self.to_enum_value(choice), readable)
            for choice, readable in flatchoices
        ]

    def formfield(self, **kwargs):
        """
        Uses `django.forms.Field`'s parameter generation with
        `EnumChoiceField` specific updates.
        """

        defaults = {
            'required': not self.blank,
            'label': capfirst(self.verbose_name),
            'help_text': self.help_text,
            'enum_class': self.enum_class,
            'choice_builder': self.choice_builder
        }

        if self.has_default():
            if callable(self.default):
                defaults['initial'] = self.default
                defaults['show_hidden_initial'] = True
            else:
                defaults['initial'] = self.get_default()

        include_blank = (self.blank or
                         not (self.has_default() or 'initial' in kwargs))
        defaults['choices'] = self.get_choices(include_blank=include_blank)

        # Many of the subclass-specific formfield arguments (min_value,
        # max_value) don't apply for choice fields, so be sure to only pass
        # the values that TypedChoiceField will understand.
        for k in list(kwargs):
            if k not in (
                'coerce', 'choices', 'required', 'enum_class', 'disabled',
                'choice_builder, ''widget', 'label', 'initial', 'help_text',
                'error_messages', 'show_hidden_initial',
            ):
                del kwargs[k]

        defaults.update(kwargs)

        return EnumChoiceFormField(**defaults)



# Generated by Django 2.2.2 on 2019-08-24 17:31

from django.db import migrations, models
import django_enum_choices.choice_builders
import django_enum_choices.fields
import test_models.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SomeModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('choices', django_enum_choices.fields.EnumChoiceField(choice_builder=django_enum_choices.choice_builders.value_value, choices=[('a', 'a'), ('aa', 'aa'), ('aaa', 'aaa')], enum_class=test_models.models.ChoicesA, max_length=3)),
                ('other_choices', django_enum_choices.fields.EnumChoiceField(choice_builder=django_enum_choices.choice_builders.value_value, choices=[('a', 'a')], enum_class=test_models.models.ChoicesB, max_length=1)),
            ],
        ),
    ]
